#include "matrix.h"
#include "fonts.h"
                                                     
volatile COLOR RGB_ARRAY[16][128]={0};   

void init_matrix(){
  RGB_PORT_DDR = 0xFF;
  DECODER_PORT_DDR = 0xFF;
  sbi(SHIFT_CLK_PORT_DDR,SHIFT_CLK_PIN);
  sbi(SHIFT_LATCH_PORT_DDR,SHIFT_LATCH_PIN);
  sbi(SHIFT_OE_PORT_DDR,SHIFT_OE_PIN);
}

void __DECODER_SET(uint8_t data){
  DECODER_PORT = data;
  delayMicroseconds(1);
}

void __SHIFT_OUT(uint8_t line){
  for(uint8_t seg=0;seg<128;seg++){
    RGB_PORT=RGB_ARRAY[line][seg+2]; 
    //RGB_PORT=0xFF;
    //RGB_PORT=(seg+line)<<2;
    sbi(SHIFT_LATCH_PORT,SHIFT_CLK_PIN);
    cbi(SHIFT_LATCH_PORT,SHIFT_CLK_PIN);
  }
    sbi(SHIFT_LATCH_PORT,SHIFT_LATCH_PIN);
    cbi(SHIFT_LATCH_PORT,SHIFT_LATCH_PIN);
}

void __DYNAMIC_IDICATION(void){
  static volatile uint16_t i=0;
    sbi(SHIFT_OE_PORT,SHIFT_OE_PIN);
    __DECODER_SET(i);
    __SHIFT_OUT(i);
    cbi(SHIFT_OE_PORT,SHIFT_OE_PIN);
    i++;
    i%=16;
}

void matrix_fill(COLOR color){
  for(int x=0;x<128;x++){
    for(int y=0;y<16;y++){
      RGB_ARRAY[y][x]=color<<2 | color<<5;
    }
  }
}

void matrix_draw_pixel(int16_t x, int16_t y, COLOR color){                
  if(x<0 or x>127)return;
  if(y<0 or y>31)return;
  y/16?RGB_ARRAY[y%16][x]&=0b00011111:RGB_ARRAY[y%16][x]&=0b11100011;
  RGB_ARRAY[y%16][x]|=color<<(2+(y/16*3));
}

void matrix_draw_print_char_7x5(int16_t x, int16_t y, uint8_t color, char _char){
  if(x+(int16_t)6<0 || x>127)return;
  for(int16_t _y=0; _y<7; _y++){
    for(int16_t _x=0; _x<5; _x++){
      matrix_draw_pixel(x+_x,y+_y,color*((ascii[(uint8_t)_char][_y]&(1<<(4-_x)))>>(4-_x)));
    }
  }
}

void matrix_draw_print_7x5(int16_t x, int16_t y, uint8_t color, char* text){
  for(uint8_t _x=0;_x<255;_x++){
    if(text[_x]==0x00){return;}
    if(text[_x]==0x0D){return;}
      matrix_draw_print_char_7x5(x+(_x*6), y, color, text[_x]);
      matrix_draw_print_char_7x5(x+(_x*6)+5, y, color, 0);
  }
}

void matrix_draw_print_char_7x5_scaled(int16_t x, int16_t y, uint8_t color, char _char, uint8_t scale){
  if(x+(int16_t)(6*scale)<0 || x>128)return;
  for(int16_t _y=0; _y<7; _y++){
    for(int16_t _x=0; _x<5; _x++){
      for(int16_t scale_x=0;scale_x<scale;scale_x++){
        for(int16_t scale_y=0;scale_y<scale;scale_y++){
          matrix_draw_pixel(y+(_y*scale)+scale_y,x+(_x*scale)+scale_x,color*((ascii[(uint8_t)_char][_y]&(1<<(4-_x)))>>(4-_x)));
        }
      }
    }
  }
}

void matrix_draw_print_7x5_scaled(int16_t x, int16_t y, uint8_t color, char* text, uint8_t scale){        
  scale--;
  for(uint8_t _x=0;_x<255;_x++){
    if(text[_x]==0x00){return;}
    if(text[_x]==0x0D){return;}
      matrix_draw_print_char_7x5_scaled(x+(_x*(6*scale)), y, color, text[_x], scale);
      matrix_draw_print_char_7x5_scaled(x+(_x*(6*scale))+(5*scale), y, color, 0, scale);
  }
}
